c      call example_1 ! 1-dimensional example
c      call example_2 ! 2-dimensional example
c      call example_3 ! 3-dimensional example
      call example_4 ! 2-dimensional example, VEGAS-like vs direct
      end


      subroutine example_1
*  *********************************************************************
*  * 1-dimensional integration and density-estimation example
*  *********************************************************************
      implicit none
      integer nev,nopt,ianalyse,itask,ndim,nbat_s,nbatch,iev,nunit
     &,nchmax,sampling,estimating
      parameter(nunit=21)
      double precision wght,xx(1),ave,sig,eff,rev,ww
     &,avh_parni_density,cauchy_dnst
*
* Dimension
      ndim = 1
*
* Initialize integrand
      call cauchy_init(1 ,0.6d0,0.00001d0)
*
* Initialize Monte Carlo
      nev  = 20000 ! total number of events
      nopt = 10000 ! number of events used for optimizing parni
      call avh_moncar_init(1) ! all events
      call avh_moncar_init(2) ! only events after optimization
*
* Initialize parni for integration task
      sampling = 17  ! label for parni
      itask  = 1     ! for integration the options are 1,2,3,11,12,13
      nbat_s = int(dble(nopt)**(0.5))
      nchmax = 0
      call avh_parni_init(sampling ,itask,ndim,nbat_s ,nchmax)
*
* Initialize parni for density-estimation task
      estimating = 82  ! label for parni
      itask  = 11      ! for density estimation, the only option is 11
      nbatch = int(dble(nopt)**(0.5))
      nchmax = 100
      call avh_parni_init(estimating ,itask,ndim,nbatch ,nchmax)
*
* Start MC-loop
      do iev=1,nev
*
* Integration example:
*
* * generate x
        call avh_parni_generate(sampling,xx)
*
* * calculate the integrand
        wght = cauchy_dnst(1 ,xx)
*
* * calculate the MC-weight
        wght = wght/avh_parni_density(sampling,xx)
*
* * adapt PARNI, only during the first nopt events
        if (iev.le.nopt) call avh_parni_adapt(sampling,wght,xx)
*
* * collection of events after optimization
        if (iev.gt.nopt) call avh_moncar_collect(2,wght)
*
* * collection of events including optimization
        call avh_moncar_collect(1,wght)
*
* * during optimization, batches of events contribute to the final
* * result with weight wght
        if (iev.le.nopt.and.iev.eq.iev/nbat_s*nbat_s) then
          wght = ( dble(iev)/dble(nopt) )**(1d0)
          wght = min(1d0,wght)
          call avh_moncar_batch(1 ,wght)
        endif
*
* Density-estimation example:
*
* * Generate x from an "unknown" distribution
        call cauchy_gnrt(1 ,xx)
*
* * Feed x to PARNI with weight 1
        call avh_parni_adapt(estimating,1d0,xx)
*
* * Put the bins to zero after optimization but keep the cell-structure
c        if (iev.eq.nopt) call avh_parni_clean(estimating)
*
* End MC-loop
      enddo
*
* Results of integration:
* Include the final batch of events for the result including
* optimization, and return this result
      call avh_moncar_batch(1 ,dble(nev-nopt)/nbat_s)
      call avh_moncar_result(1,ave,sig,eff,rev)
      write(6,*)
      write(6,102) 'example_1:'
     &            ,'estimate +/- error','Nevents','efficiency'
      write(6,101) '    total:',ave,' +/-',sig,rev,eff
*
* Return the result only including events after the optimization process
      call avh_moncar_result(2,ave,sig,eff,rev)
      write(6,101) 'optimized:',ave,' +/-',sig,rev,eff
      write(6,*)
      call avh_parni_plot(sampling,nunit,0) ! plot distribution
      call avh_parni_marg(sampling,nunit,0) ! plot distribution
      call avh_parni_result(sampling)       ! optimization diagnostics
*
* Results of density-estimation:
      call avh_parni_plot(estimating,nunit,0) ! plot distribution
      call avh_parni_marg(estimating,nunit,0) ! plot distribution
      call avh_parni_result(estimating)       ! optimization diagnostics
*
  101 format(a10,f11.8,a4,f11.8,f11.0,f12.8)
  102 format(a10,a21,a16,a12)
      end


      subroutine example_2
*  *********************************************************************
*  * 2-dimensional integration and density-estimation example
*  *********************************************************************
      implicit none
      integer nev,nopt,ianalyse,itask,ndim,nbat_s,nbatch,iev,nunit
     &,nchmax,sampling,estimating
      parameter(nunit=21)
      double precision wght,xx(2),ave,sig,eff,rev,ww
     &,avh_parni_density,cirkel_dnst,diagonal_dnst
*
* Dimension
      ndim = 2
*
* Initialize integrand
      call cirkel_init(0.57d0,0.62d0 ,0.3d0,0.01d0)
*
* Initialize Monte Carlo
      nev  = 200000 ! total number of events
      nopt = 100000 ! number of events used for optimizing parni
      call avh_moncar_init(1)
      call avh_moncar_init(2)
*
* Initialize parni for integration task
      sampling = 13  ! label for parni
      itask  = 1     ! for integration the options are 1,2,3,11,12,13
      nbat_s = int(dble(nopt)**(0.5))
      nchmax = 00
      call avh_parni_init(sampling ,itask,ndim,nbat_s ,nchmax)
*
* Initialize parni for density-estimation task
      estimating = 12  ! label for parni
      itask  = 11      ! for density estimation, the only option is 11
      nbatch = int(dble(nopt)**(0.5))
      nchmax = 00
      call avh_parni_init(estimating ,itask,ndim,nbatch ,nchmax)
*
* Start MC-loop
      do iev=1,nev
*
* Integration example:
*
* * generate x
        call avh_parni_generate(sampling,xx)
*
* * calculate the integrand
        wght = cirkel_dnst(xx(1),xx(2))
*
* * calculate the MC-weight
c        wght = wght/avh_parni_density(sampling,xx)
        call avh_parni_weight(sampling ,ww,xx)
        wght = wght*ww
*
* * adapt PARNI, only during the first nopt events
        if (iev.le.nopt) call avh_parni_adapt(sampling,wght,xx)
*
* * collection of events after optimization
        if (iev.gt.nopt) call avh_moncar_collect(2,wght)
*
* * collection of events including optimization
        call avh_moncar_collect(1,wght)
*
* * during optimization, batches of events contribute to the final
* * result with weight wght
        if (iev.le.nopt.and.iev.eq.iev/nbat_s*nbat_s) then
          wght = ( dble(iev)/dble(nopt) )**(1d0)
          wght = min(1d0,wght)
          call avh_moncar_batch(1 ,wght)
        endif
*
* Density-estimation example:
*
* * Generate x from an "unknown" distribution
        call cirkel_gnrt(xx(1),xx(2))
*
* * Feed x to PARNI with weight 1
        call avh_parni_adapt(estimating,1d0,xx)
*
* * Put the bins to zero after optimization but keep the cell-structure
        if (iev.eq.nopt) call avh_parni_clean(estimating)
*
* End MC-loop
      enddo
*
* Results of integration:
* Include the final batch of events for the result including
* optimization, and return this result
      call avh_moncar_batch(1 ,dble(nev-nopt)/nbat_s)
      call avh_moncar_result(1,ave,sig,eff,rev)
      write(6,*)
      write(6,102) 'example_2:'
     &            ,'estimate +/- error','Nevents','efficiency'
      write(6,101) '    total:',ave,' +/-',sig,rev,eff
*
* Return the result only including events after the optimization process
      call avh_moncar_result(2,ave,sig,eff,rev)
      write(6,101) 'optimized:',ave,' +/-',sig,rev,eff
      write(6,*)
      call avh_parni_plot(sampling,nunit,0) ! plot distribution
      call avh_parni_marg(sampling,nunit,0) ! plot marginals
      call avh_parni_result(sampling)       ! optimization diagnostics
*
* Results of density-estimation:
      call avh_parni_plot(estimating,nunit,0) ! plot distribution
      call avh_parni_marg(estimating,nunit,0) ! plot marginals
      call avh_parni_result(estimating)       ! optimization diagnostics
*
  101 format(a10,f11.8,a4,f11.8,f11.0,f12.8)
  102 format(a10,a21,a16,a12)
      end


      subroutine example_3
*  *********************************************************************
*  * 3-dimensional integration example
*  *********************************************************************
      implicit none
      integer nev,nopt,ianalyse,itask,ndim,nbat_s,nbatch,iev,nunit
     &,nchmax,sampling,estimating
      parameter(nunit=21)
      double precision wght,xx(3),ave,sig,eff,rev
     &,avh_parni_density,sphere3_dnst,sphere_dnst
*
* Dimension
      ndim = 3
*
* Initialize integrand
c     call sphere3_init( a1_in,  a2_in,  a3_in, rad_in, thi_in)
      call sphere3_init(0.25d0, 0.40d0, 0.50d0, 0.35d0, 0.02d0)
*
* Initialize Monte Carlo
      nev  = 10000000 ! total number of events
      nopt = 9000000 ! number of events used for optimizing parni
      call avh_moncar_init(1) ! all events
      call avh_moncar_init(2) ! only events after optimization
*
* Initialize parni for integration task
      sampling = 5  ! label for parni
      itask  = 1     ! for integration the options are 1,2,3,11,12,13
      nbat_s = int(dble(nopt)**(0.5))
      nchmax = 10000
      call avh_parni_init(sampling ,itask,ndim,nbat_s ,nchmax)
*
* Start MC-loop
      do iev=1,nev
*
* Integration example:
*
* * generate x
        call avh_parni_generate(sampling,xx)
*
* * calculate the integrand
        wght = sphere3_dnst(xx)
*
* * calculate the MC-weight
        wght = wght/avh_parni_density(sampling,xx)
*
* * adapt PARNI, only during the first nopt events
        if (iev.le.nopt) call avh_parni_adapt(sampling,wght,xx)
*
* * collection of events after optimization
        if (iev.gt.nopt) call avh_moncar_collect(2,wght)
*
* * collection of events including optimization
        call avh_moncar_collect(1,wght)
*
* * during optimization, batches of events contribute to the final
* * result with weight wght
        if (iev.le.nopt.and.iev.eq.iev/nbat_s*nbat_s) then
          wght = ( dble(iev)/dble(nopt) )**(1d0)
          wght = min(1d0,wght)
          call avh_moncar_batch(1 ,wght)
        endif
*
* End MC-loop
      enddo
*
* Include the final batch of events for the result including
* optimization, and return this result
      call avh_moncar_batch(1 ,dble(nev-nopt)/nbat_s)
      call avh_moncar_result(1,ave,sig,eff,rev)
      write(6,*)
      write(6,102) 'example_3:'
     &            ,'estimate +/- error','Nevents','efficiency'
      write(6,101) '    total:',ave,' +/-',sig,rev,eff
*
* Return the result only including events after the optimization process
      call avh_moncar_result(2,ave,sig,eff,rev)
      write(6,101) 'optimized:',ave,' +/-',sig,rev,eff
      write(6,*)
      call avh_parni_marg(sampling,nunit,0) ! plot marginals
      call avh_parni_result(sampling)       ! optimization diagnostics
*
  101 format(a10,f11.8,a4,f11.8,f11.0,f12.8)
  102 format(a10,a21,a16,a12)
      end


      subroutine example_4
*  *********************************************************************
*  * 2-dimensional integration example of an integrand which is 
*  * factorizable in the two dimensions. Both the factorized VEGAS-like
*  * approach and the direct approach are applied.
*  *********************************************************************
      implicit none
      integer nev,nopt,ianalyse,itask,ndim,nbat_1,nbat_3,iev,nunit
     &,nchmax,dim1,dim2,both
      parameter(nunit=21)
      double precision wght,xx(2),ave,sig,eff,rev,ww
     &,avh_parni_density,cauchy_dnst,cirkel_dnst
*
* Initialize integrand
      call cauchy_init(1 ,0.6d0,0.02d0) ! factor of the first dimension
      call cauchy_init(2 ,0.33d0,0.04d0) ! factor of the second dimension
c      call cirkel_init(0.57d0,0.62d0 ,0.3d0,0.01d0)
*
* Initialize Monte Carlo
      nev  = 200000 ! total number of events
      nopt = 100000 ! number of events used for optimizing parni
      call avh_moncar_init(1) ! VEGAS-like, including all events
      call avh_moncar_init(2) ! VEGAS-like, after optimization
      call avh_moncar_init(3) ! direct, including all events
      call avh_moncar_init(4) ! direct, after optimization
*
* Initialize parni for the first dimension in the VEGAS-approach
      dim1   = 1   ! label for parni, first dimension
      itask  = 1   ! for integration the options are 1,2,3,11,12,13
      ndim   = 1   ! number of dimensions
      nbat_1 = int(dble(nopt)**(0.5))
      nchmax = 100
      call avh_parni_init(dim1 ,itask,ndim,nbat_1 ,nchmax)
*
* Initialize parni for the second dimension in the VEGAS-approach
      dim2   = 2   ! label for parni, second dimension
      itask  = 1   ! for integration the options are 1,2,3,11,12,13
      ndim   = 1   ! number of dimensions
      nchmax = 100
      call avh_parni_init(dim2 ,itask,ndim,nbat_1 ,nchmax)
*
* Initialize parni for the direct approach
      both   = 3   ! label for parni, both dimensions at once
      itask  = 1   ! for integration the options are 1,2,3,11,12,13
      ndim   = 2   ! number of dimensions
      nbat_3 = int(dble(nopt)**(0.5))
      nchmax = 200
      call avh_parni_init(both ,itask,ndim,nbat_3 ,nchmax)
*
* Start MC-loop
      do iev=1,nev
*
* VEGAS-like approach
*
* * generate two-dimensional x
        call avh_parni_generate(dim1,xx(1))
        call avh_parni_generate(dim2,xx(2))
*
* * calculate the integrand
        wght = cauchy_dnst(1,xx(1)) * cauchy_dnst(2,xx(2))
c        wght = cirkel_dnst(xx(1),xx(2))
*
* * calculate the MC-weight
        wght = wght/avh_parni_density(dim1,xx(1))
     &             /avh_parni_density(dim2,xx(2))
*
* * adapt PARNI, only during the first nopt events
        if (iev.le.nopt) call avh_parni_adapt(dim1,wght,xx(1))
        if (iev.le.nopt) call avh_parni_adapt(dim2,wght,xx(2))
*
* * collection of events after optimization
        if (iev.gt.nopt) call avh_moncar_collect(2,wght)
*
* * collection of events including optimization
        call avh_moncar_collect(1,wght)
*
* * during optimization, batches of events contribute to the final
* * result with weight wght
        if (iev.le.nopt.and.iev.eq.iev/nbat_1*nbat_1) then
          wght = ( dble(iev)/dble(nopt) )**(1d0)
          wght = 1d0 !min(1d0,wght)
          call avh_moncar_batch(1 ,wght)
        endif
*
* Direct approach
*
* * generate two-dimensional x
        call avh_parni_generate(both,xx)
*
* * calculate the integrand
        wght = cauchy_dnst(1,xx(1)) * cauchy_dnst(2,xx(2))
c        wght = cirkel_dnst(xx(1),xx(2))
*
* * calculate the MC-weight
c        wght = wght/avh_parni_density(both,xx)
        call avh_parni_weight(both ,ww,xx)
        wght = wght*ww
*
* * adapt PARNI, only during the first nopt events
        if (iev.le.nopt) call avh_parni_adapt(both,wght,xx)
*
* * collection of events after optimization
        if (iev.gt.nopt) call avh_moncar_collect(4,wght)
*
* * collection of events including optimization
        call avh_moncar_collect(3,wght)
*
* * during optimization, batches of events contribute to the final
* * result with weight wght
        if (iev.le.nopt.and.iev.eq.iev/nbat_3*nbat_3) then
          wght = ( dble(iev)/dble(nopt) )**(1d0)
          wght = 1d0 !min(1d0,wght)
          call avh_moncar_batch(3 ,wght)
        endif
*
* * test the dump, remove and load features
!       if (iev.eq.nopt/9) write(*,*) 'Dumping  ID=',both
!       if (iev.eq.nopt/9) call avh_parni_dump( both ,21 )
!       if (iev.eq.nopt/9) write(*,*) 'Removing ID=',both
!       if (iev.eq.nopt/9) call avh_parni_remove( both )
!       if (iev.eq.nopt/9) write(*,*) 'Loading  ID=',both
!       if (iev.eq.nopt/9) call avh_parni_load( both ,21 )
*
* End MC-loop
      enddo
*
* Return results
      write(6,*)
      write(6,'(a30)') 'example_4, VEGAS-like approach'
*
* Include the final batch of events for the result including
* optimization, and return this result
      call avh_moncar_batch(1 ,dble(nev-nopt)/nbat_1)
      call avh_moncar_result(1,ave,sig,eff,rev)
      write(6,102) ' '
     &            ,'estimate +/- error','Nevents','efficiency'
      write(6,101) '    total:',ave,' +/-',sig,rev,eff
*
* Return the result only including events after the optimization process
      call avh_moncar_result(2,ave,sig,eff,rev)
      write(6,101) 'optimized:',ave,' +/-',sig,rev,eff
      call avh_parni_marg(dim1,nunit,0) ! plot distribution dimension 1
      call avh_parni_marg(dim2,nunit,0) ! plot distribution dimension 2
      call avh_parni_result(dim1) ! optimization diagnostics dimension 1
      call avh_parni_result(dim2) ! optimization diagnostics dimension 2
*
      write(6,*)
      write(6,'(a26)') 'example_4, direct approach'
*
* Include the final batch of events for the result including
* optimization, and return this result
      call avh_moncar_batch(3 ,dble(nev-nopt)/nbat_3)
      call avh_moncar_result(3,ave,sig,eff,rev)
      write(6,102) ' '
     &            ,'estimate +/- error','Nevents','efficiency'
      write(6,101) '    total:',ave,' +/-',sig,rev,eff
*
* Return the result only including events after the optimization process
      call avh_moncar_result(4,ave,sig,eff,rev)
      write(6,101) 'optimized:',ave,' +/-',sig,rev,eff
      call avh_parni_plot(both,nunit,0) ! plot distribution
      call avh_parni_marg(both,nunit,0) ! plot marginals
      call avh_parni_result(both) ! optimization diagnostics
*
  101 format(a10,f11.8,a4,f11.8,f11.0,f12.8)
  102 format(a10,a21,a16,a12)
      end



      subroutine cauchy_init(ID ,x0_in,ep_in)
*  **************************************************************
*  * Normalized density of the type
*  *               1/( (x-x0)^2 + ep^2 )
*  **************************************************************
      implicit none
      integer ID,nID
      double precision x0_in,ep_in ,xmin,xmax
      parameter(xmin=0d0,xmax=1d0)
      parameter(nID=10)
      double precision  x0(nID),ep(nID),fmin(nID),fnrm(nID)
      common/cauchy_com/x0     ,ep     ,fmin     ,fnrm
      integer iev,nev,nunit,ndig
      parameter(nev=100,nunit=21)
      double precision xx,yy,cauchy_dnst
      character*24 filename,chii
*
      x0(ID) = x0_in
      ep(ID) = ep_in
      fmin(ID) = datan( (xmin-x0(ID))/ep(ID) )/ep(ID)
      fnrm(ID) = datan( (xmax-x0(ID))/ep(ID) )/ep(ID) - fmin(ID)
*
      filename = '                        '
      filename(1:6) = 'CAUCHY'
      call avh_parni_charnum(chii,ndig ,ID)
      filename(6+1:6+ndig) = chii(1:ndig)
      write(6,*) 'MESSAGE from cauchy_init: creating file ',filename
      open(nunit,file=filename)
      do iev=0,nev
        xx = xmin + dble(iev)*(xmax-xmin)/dble(nev)
        yy = cauchy_dnst(ID ,xx)
        write(nunit,'(2d16.8)') xx,yy
      enddo
      close(nunit)
      end
*
      subroutine cauchy_gnrt(ID ,xx)

      implicit none
      integer ID,nID
      double precision xx ,rho
      parameter(nID=10)
      double precision  x0(nID),ep(nID),fmin(nID),fnrm(nID)
      common/cauchy_com/x0     ,ep     ,fmin     ,fnrm
      call avh_parni_random(rho,1)
      xx = fnrm(ID)*rho + fmin(ID)
      xx = x0(ID) + ep(ID)*dtan( ep(ID)*xx )
      end
*
      double precision function cauchy_dnst(ID ,xx)
*
      implicit none
      integer ID,nID
      double precision xx ,yy
      parameter(nID=10)
      double precision  x0(nID),ep(nID),fmin(nID),fnrm(nID)
      common/cauchy_com/x0     ,ep     ,fmin     ,fnrm
      yy = xx - x0(ID)
      cauchy_dnst = 1d0/( fnrm(ID)*(ep(ID)**2 + yy**2) )
      end


      subroutine diagonal_init(a_in,b_in)
*  *********************************************************************
*  * generate  xx,yy  in  [0,1]*[0,1]  with density
*  *             1/( (xx+yy-aa)^2 + bb^2 )/area
*  *********************************************************************
      implicit none
      double precision a_in,b_in ,a1,a2
      double precision     aa,bb,area,pi
      common/diagonal_com/ aa,bb,area,pi
      pi = 4d0*datan(1d0)
      aa = a_in
      bb = b_in
      a1 = aa-1d0
      a2 = aa-2d0
      area = -0.5d0/bb*( - 2d0*bb*dlog( bb*bb + a1*a1 )
     .                   +     bb*dlog( bb*bb + a2*a2 )
     .                   +     bb*dlog( bb*bb + aa*aa )
     .                   + 4d0*a1*datan( a1/bb )
     .                   - 2d0*a2*datan( a2/bb )
     .                   - 2d0*aa*datan( aa/bb )        )
      end
*
      subroutine diagonal_gnrt(xx,yy)
*
      implicit none
      double precision xx,yy, gx,f0,fx,hh
      double precision     aa,bb,area,pi
      common/diagonal_com/ aa,bb,area,pi
* generate  xx  using rejection  
  100 call rangen(xx,1)
      call rangen(gx,1)
      hh = (xx-aa)/bb
        gx = pi*gx
      f0 = datan(hh)
      fx = datan(1d0/bb+hh) - f0
      if (gx.gt.fx) goto 100      
* generate  yy
      call rangen(yy,1)
      yy = fx*yy + f0
      yy = dtan(yy)
      yy = bb*(yy-hh)
      end
*
      double precision function diagonal_dnst(xx,yy)
*
      implicit none
      double precision wght,xx,yy
      double precision     aa,bb,area,pi
      common/diagonal_com/ aa,bb,area,pi
      wght = area*( bb*bb + (xx+yy-aa)**2 )
      diagonal_dnst = 1d0/wght
      end


      subroutine cirkel_init(a_in ,b_in ,e_in ,s_in)
*  *********************************************************************
*  * generate  xx,yy  on cirkel around  aa,bb
*  * with radius distributed Gaussian around  ee  with width  ss
*  *********************************************************************
      implicit none
      double precision e_in,s_in,a_in,b_in
      double precision   ee,ss,aa,bb,twopi
      common/cirkel_com/ ee,ss,aa,bb,twopi
      twopi = 8d0*datan(1d0)
      ee = e_in
      ss = s_in
      aa = a_in
      bb = b_in
      end
*
      subroutine cirkel_gnrt(xx,yy)
*
      implicit none
      double precision xx,yy ,h1,h2,x1,x2,rho(2)
      integer counter,init
      double precision   ee,ss,aa,bb,twopi
      common/cirkel_com/ ee,ss,aa,bb,twopi
      data counter/0/
      save x1,x2,counter
      if (counter.eq.0) then
        call rangen(rho,2)
        h1 = -dlog(rho(1))
        h2 = twopi*rho(2)
        x1 = h1*dcos(h2)
        x2 = h1*dsin(h2)
      else
        x1 = x2
      endif
      counter = counter+1
      if (counter.eq.2) counter = 0
*
      h1 = ss*x1 + ee
      call rangen(rho,1)
      h2 = twopi*rho(1)
      xx = aa + h1*dcos(h2)
      yy = bb + h1*dsin(h2)
      end
*
      double precision function cirkel_dnst(xx,yy)
*
      implicit none
      double precision wght,xx,yy ,h1,h2,rr
      double precision   ee,ss,aa,bb,twopi
      common/cirkel_com/ ee,ss,aa,bb,twopi
      rr = dsqrt( (xx-aa)**2 + (yy-bb)**2 )
      h1 = ss*ss
      h2 = 2*h1
      wght = dexp( -((rr-ee)**2)/h2 ) !+ dexp( -((rr+ee)**2)/h2 )
      wght = wght/( twopi*dsqrt(twopi*h1)*rr )
      cirkel_dnst = wght
      end


      subroutine sphere3_init(a1_in ,a2_in ,a3_in ,rad_in ,thi_in)
*  ********************************************************************
*  ********************************************************************
      implicit none
      double precision a1_in,a2_in,a3_in,rad_in,thi_in
      double precision   a1,a2,a3,rad,thi,pi
      common/sphere3_com/ a1,a2,a3,rad,thi,pi
      a1 = a1_in
      a2 = a2_in
      a3 = a3_in
      rad = rad_in
      thi = thi_in
      pi = 4*datan(1d0)
      end
*
      double precision function sphere3_dnst(xx)
*
      implicit none
      double precision xx(3) ,rr,ff
      double precision   a1,a2,a3,rad,thi,pi
      common/sphere3_com/ a1,a2,a3,rad,thi,pi
      rr = dsqrt( (xx(1)-a1)**2 + (xx(2)-a2)**2 + (xx(3)-a3)**2 )
      ff = thi/( (rr-rad)**2 + thi**2 )/pi
      ff = ff/(4d0*pi*rad**2)
      sphere3_dnst = ff
      end


      subroutine sphere_init(ndim_in)
*  ********************************************************************
*  ********************************************************************
      implicit none
      integer ndim_in ,ii
      double precision   pos(100),rad,gam,spi
      integer                                 ndim
      common/sphere_com/ pos     ,rad,gam,spi,ndim
      ndim = ndim_in
      do ii=1,ndim,2
        pos(ii  ) = 0.25d0
        pos(ii+1) = 0.40d0
      enddo
      rad = 0.35d0
      gam = 0.02d0
      spi = dsqrt( 4*datan(1d0) )
      end
*
      double precision function sphere_dnst(xx)
*
      implicit none
      double precision xx(100) ,rr,ff
      integer ii
      double precision   pos(100),rad,gam,spi
      integer                                ndim
      common/sphere_com/ pos     ,rad,gam,spi,ndim
      rr = 0d0
      do ii=1,ndim
        rr = rr + ( xx(ii) - pos(ii) )**2
      enddo
      rr = dsqrt(rr)
      ff = gam/( (rr-rad)**2 + gam**2 )/spi
c      ff = dexp( -(rr/gam)**2 )/gam/spi
      sphere_dnst = ff
      end
